#include <Wire.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 16, 2);
int schnappPins[] = {5, 8, 12};//pins für schnappschalter unter den Töpfen
#define lampPin 4 // Lampe
#define ptPin A1 //PT-PIN
int moisturePins[] = {A0, A2, A3}; // Feuchtigkeitssensoren
#define magnetVentil 7 //Magnetventil
#define piezoPin 9 //Piezo 
#define wasserstandPin A6 //Wasserstand 
#define directionPin 3 // Stepper-Motor direction Pin
#define stepPin 11 // Stepper-Motor step Pin

int pos = 1; // aktuelle Position des Gießschlauchs: 0->Topf1, 1->Topf2, 2->Topf3 
int ziel = 2; // da wo die schiene hinfahren muss
int STEPS_BETWEEN_POTS = 800; // Anzahl der Schritte zwischen zwei benachbarten Töpfen
//led
unsigned long zaehlen;
unsigned long zaehlenSek;
int state = -1;
int imSchattenZeit = 3; //Solange darf die Pflanze im Schatten stehen (in Sekunden)
unsigned long LichtAnZeit = 20;  //Solange bleibt die Lampe an (in Sekunden)
unsigned long LichtCooldown = 5;  //Nachdem die Lampe an ist, muss sie solange ausbleiben (in Sekunden)

int MOISTURE_THRESHOLD = 500; // über diesen Wert gilt es als zu trocken


void setup() {
  Serial.begin(9600);
  pinMode(lampPin, OUTPUT); //Hier LED 
  pinMode(magnetVentil, OUTPUT); //MagnetVentil
  //schiene
  pinMode(directionPin, OUTPUT); // direction pin bei unsere schaltung. 
  pinMode(stepPin, OUTPUT);// bewegungspin
  //ausgabe Monitor
  lcd.init();
  lcd.backlight();
}

void loop() {

  boolean topfDa[] = {false, false, false}; // ist der entsprechende Topf überhaupt da?
  
  lcd.clear();
  //feuchtigkeitssensoren
 /* int moisture1Value = measure1Moisture();
  int moisture2Value = measure2Moisture();
  int moisture3Value = measure3Moisture();*/

  /* AUSGABE, WELCHE TÖPFE DA SIND: */
  lcd.setCursor(0, 0);
  lcd.print("Topf  1   2   3 ");
  
  for (int i = 0; i < 3; i++) {
    
    topfDa[i] = digitalRead(schnappPins[i]); /* Gibt es diesen Topf überhaupt?
     vorausgesetzt HIGH steht für "Topf ist da", LOW für "kein Topf"
     ansonsten umkehren: topfDa[i] = !digitalRead(schnappPins[i]);*/
    setCursorPosition(i);
    if (topfDa) {
      lcd.print(" J ");
    } else {
      lcd.print(" N ");
    }
  }

  delay(1000); //Lesezeit
  
  /* FEUCHTIGKEIT AUSGEBEN: */
  lcd.setCursor(0, 0);
  lcd.print("Moist 1   2   3 ");

  for (int i = 0; i < 3; i++) {
    setCursorPosition(i);

    if (topfDa[i]) {
      int moistureValue = measureMoisture(moisturePins[i]);
      if (moistureValue > MOISTURE_THRESHOLD) {
        lcd.print("dry");
      } else {
        lcd.print("wet");
      }
    } else {
      lcd.print("N/A"); // es gibt keinen Topf
    }
  }

  /*FEUCHTIGKEIT MESSEN, SCHIENE BEWEGEN, WASSER GEBEN: */
  for (int i = 0; i < 3; i++) {
    setCursorPosition(i);

    if (topfDa[i]) {
      int moistureValue = measureMoisture(moisturePins[i]);

      while (moistureValue > MOISTURE_THRESHOLD) {
        lcd.print("dry");
        moveRailFromTo(pos, i); // Schiene bewegen
        pos = i; // aktuelle Position neu setzen
        waterGo(); // Ventil öffnen und schließen
        checkWaterInBottle(); // Wasserverfügbarkeit überprüfen
        moistureValue = measureMoisture(moisturePins[i]); // Feuchtigkeitswert aktualisieren
      }

      // Wenn wir aus der Schleife raus sind, dann ist der Boden feucht.
      lcd.print("wet");

    }
  }

  delay(1000); // Zeit zum Lesen

  //LED
  unsigned long myTime;
  unsigned long myTimeSek;
  int messung = analogRead(A1); //A0 Pin wird gelesen bzw Phototransistor
  if (state == 0 || state == -1) {
    Serial.print("Helligkeit: ");
    Serial.println(messung);
  }
  myTime = millis();  //Laufzeit des Programms wird gezaehlt
  myTimeSek = myTime / 1000;  //Laufzeit wird durch 1000 geteilt, damit es in Sekunden ist
  Serial.print("Laufzeit des Programms: ");
  Serial.println(Timer(myTimeSek));
  if (state != 1 && state != 2) { //Solange das Licht an ist oder der Cooldown laeuft, interessiert es nicht wie hell die Umgebung ist
    if (messung <= 500) { //Wenn es dunkel ist, dann wird das ausgefuehrt
      state = 0;
    }
    else {  //Wenn es hell ist, dann geht es zum default state ueber und der Zaehler wird zurueckgesetzt
      state = -1;
      zaehlen = 0;
    }
  }

  switch (state) {
    case 0: //Dieser Status tritt ein, wenn das Licht aus und es dunkel ist
      lcd.clear();
      Serial.print("Zeit im Schatten: ");
      lcd.setCursor(0, 0);
      lcd.print("Zeit im Schatten");
      lcd.setCursor(0,1);
      lcd.print(zaehlen);
      lcd.setCursor(6,1);
      lcd.print("Sekunden ");
      delay(1000);
      Serial.println(Timer(zaehlen));
      if (zaehlen >= imSchattenZeit) { //Wenn es fuer eine bestimmte Anzahl an Sekunden dunkel ist, schalte das Licht an und setze den Zaehler zurueck
        digitalWrite(lampPin, HIGH);
        zaehlen = 0;
        state = 1;
        break;
      }
      else {
        zaehlen += 1;
        break;
      }
    case 1: //Das Licht ist an und die Dauer wird geprintet
      zaehlen += 1;
      lcd.clear();
      Serial.print("Licht an seit: ");
      lcd.setCursor(0,0);
      lcd.print("Licht an seit:  ");
      lcd.setCursor(0,1);
      lcd.print(zaehlen);
      lcd.setCursor(6,1);
      lcd.print("Sekunden ");
      Serial.println(Timer(zaehlen));
      if (zaehlen >= LichtAnZeit) { //Wenn es eine bestimmte Anzahl an Sekunden erreicht, schaltet sich das Licht aus
        digitalWrite(lampPin, LOW);
        state = 2;
        zaehlen = LichtCooldown;
      }
      break;
    case 2: //Das Licht ist aus und kann fuer eine bestimmte Zeit nicht angehen(Cooldown)
      Serial.print("Lichtcooldown noch: ");
      Serial.println(Timer(zaehlen));
      if (zaehlen > 0) { //Wenn der Cooldown abgelaufen ist, gehe in den default state ueber
        zaehlen -= 1;
      }
      else {
        state = -1;
      }
    default:  //Das Licht ist aus und messung ist ueber 500
      break;
  }
 delay(1000);
}

/*
   Misst Feuchtigkeit mit dem Sensor, der am übergebenen Pin angeschlossen ist,
   und gibt den Durchschnitt von 100 Messungen zurück
*/
float measureMoisture(int pin) {
  float measureSum = 0;
  for (int i = 0; i < 100; i++) {
    measureSum = measureSum + analogRead(pin);
    /*hier musst du Darya mal fragen ob das so richtig ist 
    ich glaube nämlich, dass wenn man den Code so schreibt nicht alle pins
    erreicht werden , man muss also entweder die Kabel an andere Pins anschließen
    oder die  Funktion erweitern .  da es ja pin 0, 2 und 3 ist */
    delay(1);
  }
  if (pin == moisturePins[2]) { // Besondere Umrechnung am 3. Feuchtigkeitssensor
    return map(measureSum / 100.0, 319, 635, 150, 1023);
  } else {
    return measureSum / 100.0;
  }
}

//hier wird die Zeit umgerechnet das man es besser als user erkennen kann . Und nicht nur Sekunden ausgegeben werden
String Timer(unsigned long _Sek) {
  unsigned long Sek = _Sek;
  int Min = Sek / 60;
  int Std = Min / 60;
  int Tag = Std / 24;
  Sek -= Min * 60;
  Min -= Std * 60;
  Std -= Tag * 24;
  String Time = "";
  if (Tag > 0) {
    Time += String(Tag) + " Tag(e) ";
  }
  if (Std > 0) {
    Time += String(Std) + " Stunde(n) ";
  }
  if (Min > 0) {
    Time += String(Min) + " Minute(n) ";
  }
  Time += String(Sek) + " Sekunde(n) ";
  return Time;
}

//die Funktion für das öffnen des Magnetventils
void waterGo() {
  digitalWrite(7, HIGH);
  delay(1000);
  digitalWrite(7, LOW);
}

/*
   Misst und gibt den aktuellen Wasserstand zurück
*/
float wasserstandMessung() {
  float wasserstandWert = analogRead(wasserstandPin);
  return wasserstandWert;
}

/**
   Überprüft, ob genug Wasser in der Flasche ist. Wenn ja, passiert nichts.
   Wenn nicht genug, lautet der Piezo-Lautsprecher.
*/
void checkWaterInBottle() {
  int wasserstand = wasserstandMessung();
  Serial.print("Wasserstand: ");
  Serial.println(wasserstand);
  Serial.println("---------------------------------");
  if (wasserstand >= 100) {
    tone(piezoPin, 150);
    delay(2000);
    noTone(piezoPin);
    delay(2000); // braucht man dieses Delay?
  }

}

/*
   Bewegt die Schiene in die vorgegebene Richtung
   (HIGH: Topf3->Topf2->Topf1, LOW: Topf1->Topf2->Topf3),
   indem eine vorgegebene Anzahl der Schritte vom Stepper gemacht werden
*/
void moveRail(boolean dir, int numberOfSteps)  {
  digitalWrite(directionPin, dir);
  for (int k = 0; k <= numberOfSteps ; k++) {
    digitalWrite(stepPin, HIGH);
    delay(2);
    digitalWrite(stepPin, LOW);
    delay(2);
  }

  delay(10);
}

/*
   Bewegt die Schiene von einem Topf zum anderen
   start: Topf, wo wir gerade sind
   target: Topf, wo die Schiene sich hinbewegen soll
*/
void moveRailFromTo(int start, int target) {
  int difference = start - target;
  if (difference > 0) {
    moveRail(HIGH, STEPS_BETWEEN_POTS * difference);
  } else if (difference < 0) {
    moveRail(LOW, STEPS_BETWEEN_POTS * difference * -1);
  } else {
    // wenn Start und Ziel gleich sind, soll nichts passieren
  }

}
/**
   Setzt den LCD-Zeiger auf eine bestimmte Position je nach dem,
   auf welchen Topf die Ausgabe sich bezieht
   pot=0 -> Topf1, pot=1 -> Topf2, pot=2 -> Topf3
*/
void setCursorPosition(int pot) {
  lcd.setCursor(5 + pot * 4, 1);
}
